@@ -1,20 +1,20 @@
 package POE::Component::Metabase::Client::Submit;
+$POE::Component::Metabase::Client::Submit::VERSION = '0.14';
+#ABSTRACT: a POE client that submits to Metabase servers
 
 use strict;
 use warnings;
 use Carp ();
 use HTTP::Status qw[:constants];
 use HTTP::Request::Common ();
+use HTTP::Message 5.814 (); # for HTTP::Message::decodable() support
 use JSON ();
 use POE qw[Component::Client::HTTP Component::Client::Keepalive];
 use URI;
-use vars qw[$VERSION];
-
-$VERSION = '0.12';
 
 my @valid_args;
 BEGIN {
-  @valid_args = qw(profile secret uri fact event session http_alias context resolver);
+  @valid_args = qw(profile secret uri fact event session http_alias context resolver compress);
 
   for my $arg (@valid_args) {
     no strict 'refs';
@@ -109,14 +109,29 @@ sub _submit {
     unless $fact->creator;
 
   my $req_uri = $self->_abs_uri($path);
+  my $can_decode = HTTP::Message::decodable;
 
   my $req = HTTP::Request::Common::POST(
     $req_uri,
+    ( length $can_decode ? ('Accept-Encoding' => $can_decode) : () ),
     Content_Type => 'application/json',
     Accept       => 'application/json',
     Content      => JSON->new->encode($fact->as_struct),
   );
   $req->authorization_basic($self->profile->resource->guid, $self->secret->content);
+
+  # Compress it?
+  if ( defined $self->compress and $self->compress ne 'none' ) {
+    my $err;
+    eval { $req->encode( $self->compress ) };
+    if ( $@ ) {
+      $self->{_error} = "Compression error: $@";
+      $self->{success} = 0;
+      $kernel->yield( '_dispatch' );
+      return;
+    }
+  }
+
   $kernel->yield( '_http_req', $req, 'submit' );
   return;
 }
@@ -132,15 +147,18 @@ sub _guid_exists {
 
 sub _register {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
-  my $req_uri = $self->_abs_uri('register');
 
   for my $type ( qw/profile secret/ ) {
     $self->$type->set_creator( $self->$type->resource )
       unless $self->$type->creator;
   }
 
+  my $req_uri = $self->_abs_uri('register');
+  my $can_decode = HTTP::Message::decodable;
+
   my $req = HTTP::Request::Common::POST(
     $req_uri,
+    ( length $can_decode ? ('Accept-Encoding' => $can_decode) : () ),
     Content_Type => 'application/json',
     Accept       => 'application/json',
     Content      => JSON->new->encode([
@@ -199,7 +217,17 @@ sub _response {
   else {
     $self->{success} = 1;
   }
-  $self->{content} = $res->content;
+
+  # decode the content if we requested it
+  if ( defined $request_packet->[0]->header( 'Accept-Encoding' ) ) {
+    eval { $self->{content} = $res->decoded_content( 'charset' => 'none' ) };
+    if ( $@ ) {
+      $self->{_error} = "unable to decode content: $@";
+      $self->{success} = 0;
+    }
+  } else {
+    $self->{content} = $res->content;
+  }
   $kernel->yield( '_dispatch' );
   return;
 }
@@ -266,12 +294,21 @@ sub _error {
 }
 
 'Submit this';
+
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 POE::Component::Metabase::Client::Submit - a POE client that submits to Metabase servers
 
+=head1 VERSION
+
+version 0.14
+
 =head1 SYNOPSIS
 
   use strict;
@@ -310,6 +347,8 @@ POE::Component::Metabase::Client::Submit - a POE client that submits to Metabase
 POE::Component::Metabase::Client::Submit provides a L<POE> mechanism for submitting facts
 to a L<Metabase> web server.
 
+=for Pod::Coverage profile secret uri fact event session http_alias context resolver compress
+
 =head1 CONSTRUCTOR
 
 =over
@@ -335,6 +374,7 @@ And some optional arguments:
  'http_alias', the alias or ID of an existing POE::Component::Client::HTTP session to use.
  'context', anything that can be stored in a scalar that is meaningful to you.
  'resolver', a reference to a POE::Component::Resolver object to use (ignored if http_alias is used).
+ 'compress', a compressor to use - defaults to none ( available compressors: gzip, deflate, x-bzip2, none )
 
 =back
 
@@ -365,35 +405,35 @@ This will contain the content of any HTTP responses whether success or failure.
 
 =back
 
-=head1 AUTHORS
-
-  David A. Golden (DAGOLDEN)
-
-  Ricardo SIGNES (RJBS)
-
-  Chris Williams (BINGOS)
-
-=head1 COPYRIGHT AND LICENSE
-
-  Portions Copyright (c) 2008 by Ricardo SIGNES
-  Portions Copyright (c) 2009-2010 by David A. Golden
-  Portions Copyright (c) 2010 by Chris Williams
-
-Licensed under the same terms as Perl itself (the "License").
-You may not use this file except in compliance with the License.
-A copy of the License was distributed with this file or you may obtain a
-copy of the License from http://dev.perl.org/licenses/
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
 =head1 SEE ALSO
 
 L<Metabase>
 
 L<POE>
 
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Chris Williams <chris@bingosnet.co.uk>
+
+=item *
+
+Ricardo SIGNES
+
+=item *
+
+David A. Golden
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2017 by Chris Williams.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
